@@ -189,10 +189,12 @@ class Gem::Installer
     @package.prog_mode = options[:prog_mode]
     @package.data_mode = options[:data_mode]
 
-    if options[:user_install]
-      @gem_home = Gem.user_dir
-      @bin_dir = Gem.bindir gem_home unless options[:bin_dir]
-      @plugins_dir = Gem.plugindir(gem_home)
+    if @gem_home == Gem.user_dir
+      # If we get here, then one of the following likely happened:
+      # - `--user-install` was specified
+      # - `Gem::PathSupport#home` fell back to `Gem.user_dir`
+      # - GEM_HOME was manually set to `Gem.user_dir`
+
       check_that_user_bin_dir_is_in_path
     end
   end
@@ -222,31 +224,32 @@ class Gem::Installer
 
     File.open generated_bin, "rb" do |io|
       line = io.gets
-      shebang = /^#!.*ruby/
+      shebang = /^#!.*ruby/o
 
-      if load_relative_enabled?
-        until line.nil? || line =~ shebang do
+      # TruffleRuby uses a bash prelude in default launchers
+      if load_relative_enabled? || RUBY_ENGINE == "truffleruby"
+        until line.nil? || shebang.match?(line) do
           line = io.gets
         end
       end
 
-      next unless line =~ shebang
+      next unless line&.match?(shebang)
 
       io.gets # blankline
 
-      # TODO detect a specially formatted comment instead of trying
+      # TODO: detect a specially formatted comment instead of trying
       # to find a string inside Ruby code.
-      next unless io.gets.to_s.include?("This file was generated by RubyGems")
+      next unless io.gets&.include?("This file was generated by RubyGems")
 
       ruby_executable = true
-      existing = io.read.slice(%r{
+      existing = io.read.slice(/
           ^\s*(
             gem \s |
             load \s Gem\.bin_path\( |
             load \s Gem\.activate_bin_path\(
           )
           (['"])(.*?)(\2),
-        }x, 3)
+        /x, 3)
     end
 
     return if spec.name == existing
@@ -316,7 +319,7 @@ class Gem::Installer
     FileUtils.rm_rf spec.extension_dir
 
     dir_mode = options[:dir_mode]
-    FileUtils.mkdir_p gem_dir, :mode => dir_mode && 0755
+    FileUtils.mkdir_p gem_dir, mode: dir_mode && 0o755
 
     if @options[:install_as_default]
       extract_bin
@@ -348,34 +351,30 @@ class Gem::Installer
     run_post_install_hooks
 
     spec
-
-  # TODO This rescue is in the wrong place. What is raising this exception?
-  # move this rescue to around the code that actually might raise it.
-  rescue Zlib::GzipFile::Error
-    raise Gem::InstallError, "gzip error installing #{gem}"
+  rescue Errno::EACCES => e
+    # Permission denied - /path/to/foo
+    raise Gem::FilePermissionError, e.message.split(" - ").last
   end
 
   def run_pre_install_hooks # :nodoc:
     Gem.pre_install_hooks.each do |hook|
-      if hook.call(self) == false
-        location = " at #{$1}" if hook.inspect =~ /[ @](.*:\d+)/
+      next unless hook.call(self) == false
+      location = " at #{$1}" if hook.inspect =~ /[ @](.*:\d+)/
 
-        message = "pre-install hook#{location} failed for #{spec.full_name}"
-        raise Gem::InstallError, message
-      end
+      message = "pre-install hook#{location} failed for #{spec.full_name}"
+      raise Gem::InstallError, message
     end
   end
 
   def run_post_build_hooks # :nodoc:
     Gem.post_build_hooks.each do |hook|
-      if hook.call(self) == false
-        FileUtils.rm_rf gem_dir
+      next unless hook.call(self) == false
+      FileUtils.rm_rf gem_dir
 
-        location = " at #{$1}" if hook.inspect =~ /[ @](.*:\d+)/
+      location = " at #{$1}" if hook.inspect =~ /[ @](.*:\d+)/
 
-        message = "post-build hook#{location} failed for #{spec.full_name}"
-        raise Gem::InstallError, message
-      end
+      message = "post-build hook#{location} failed for #{spec.full_name}"
+      raise Gem::InstallError, message
     end
   end
 
@@ -395,7 +394,7 @@ class Gem::Installer
       specs = []
 
       Gem::Util.glob_files_in_dir("*.gemspec", File.join(gem_home, "specifications")).each do |path|
-        spec = Gem::Specification.load path.tap(&Gem::UNTAINT)
+        spec = Gem::Specification.load path
         specs << spec if spec
       end
 
@@ -465,6 +464,9 @@ class Gem::Installer
   ##
   # Writes the full .gemspec specification (in Ruby) to the gem home's
   # specifications/default directory.
+  #
+  # In contrast to #write_spec, this keeps file lists, so the `gem contents`
+  # command works.
 
   def write_default_spec
     Gem.write_binary(default_spec_file, spec.to_ruby)
@@ -491,12 +493,11 @@ class Gem::Installer
     ensure_writable_dir @bin_dir
 
     spec.executables.each do |filename|
-      filename.tap(&Gem::UNTAINT)
       bin_path = File.join gem_dir, spec.bindir, filename
       next unless File.exist? bin_path
 
       mode = File.stat(bin_path).mode
-      dir_mode = options[:prog_mode] || (mode | 0111)
+      dir_mode = options[:prog_mode] || (mode | 0o111)
 
       unless dir_mode == mode
         require "fileutils"
@@ -524,6 +525,8 @@ class Gem::Installer
     else
       regenerate_plugins_for(spec, @plugins_dir)
     end
+  rescue ArgumentError => e
+    raise e, "#{latest.name} #{latest.version} #{spec.name} #{spec.version}: #{e.message}"
   end
 
   ##
@@ -531,7 +534,7 @@ class Gem::Installer
   #--
   # The Windows script is generated in addition to the regular one due to a
   # bug or misfeature in the Windows shell's pipe.  See
-  # http://blade.nagaokaut.ac.jp/cgi-bin/scat.rb/ruby/ruby-talk/193379
+  # https://blade.ruby-lang.org/ruby-talk/193379
 
   def generate_bin_script(filename, bindir)
     bin_script_path = File.join bindir, formatted_program_filename(filename)
@@ -539,9 +542,9 @@ class Gem::Installer
     require "fileutils"
     FileUtils.rm_f bin_script_path # prior install may have been --no-wrappers
 
-    File.open bin_script_path, "wb", 0755 do |file|
+    File.open bin_script_path, "wb", 0o755 do |file|
       file.print app_script_text(filename)
-      file.chmod(options[:prog_mode] || 0755)
+      file.chmod(options[:prog_mode] || 0o755)
     end
 
     verbose bin_script_path
@@ -566,7 +569,7 @@ class Gem::Installer
       File.unlink dst
     end
 
-    FileUtils.symlink src, dst, :verbose => Gem.configuration.really_verbose
+    FileUtils.symlink src, dst, verbose: Gem.configuration.really_verbose
   rescue NotImplementedError, SystemCallError
     alert_warning "Unable to use symlinks, installing wrapper"
     generate_bin_script filename, bindir
@@ -589,7 +592,7 @@ class Gem::Installer
 
   def shebang(bin_file_name)
     path = File.join gem_dir, spec.bindir, bin_file_name
-    first_line = File.open(path, "rb") {|file| file.gets } || ""
+    first_line = File.open(path, "rb", &:gets) || ""
 
     if first_line.start_with?("#!")
       # Preserve extra words on shebang line, like "-w".  Thanks RPA.
@@ -631,7 +634,6 @@ class Gem::Installer
 
   def ensure_loadable_spec
     ruby = spec.to_ruby_for_cache
-    ruby.tap(&Gem::UNTAINT)
 
     begin
       eval ruby
@@ -652,32 +654,37 @@ class Gem::Installer
 
   def process_options # :nodoc:
     @options = {
-      :bin_dir => nil,
-      :env_shebang => false,
-      :force => false,
-      :only_install_dir => false,
-      :post_install_message => true,
+      bin_dir: nil,
+      env_shebang: false,
+      force: false,
+      only_install_dir: false,
+      post_install_message: true,
     }.merge options
 
     @env_shebang         = options[:env_shebang]
     @force               = options[:force]
     @install_dir         = options[:install_dir]
-    @gem_home            = options[:install_dir] || Gem.dir
-    @plugins_dir         = Gem.plugindir(@gem_home)
+    @user_install        = options[:user_install]
     @ignore_dependencies = options[:ignore_dependencies]
     @format_executable   = options[:format_executable]
     @wrappers            = options[:wrappers]
     @only_install_dir    = options[:only_install_dir]
 
-    # If the user has asked for the gem to be installed in a directory that is
-    # the system gem directory, then use the system bin directory, else create
-    # (or use) a new bin dir under the gem_home.
-    @bin_dir             = options[:bin_dir] || Gem.bindir(gem_home)
+    @bin_dir             = options[:bin_dir]
     @development         = options[:development]
     @build_root          = options[:build_root]
 
     @build_args = options[:build_args]
 
+    @gem_home = @install_dir || user_install_dir || Gem.dir
+
+    # If the user has asked for the gem to be installed in a directory that is
+    # the system gem directory, then use the system bin directory, else create
+    # (or use) a new bin dir under the gem_home.
+    @bin_dir ||= Gem.bindir(@gem_home)
+
+    @plugins_dir = Gem.plugindir(@gem_home)
+
     unless @build_root.nil?
       @bin_dir = File.join(@build_root, @bin_dir.gsub(/^[a-zA-Z]:/, ""))
       @gem_home = File.join(@build_root, @gem_home.gsub(/^[a-zA-Z]:/, ""))
@@ -711,12 +718,11 @@ class Gem::Installer
   end
 
   def verify_gem_home # :nodoc:
-    FileUtils.mkdir_p gem_home, :mode => options[:dir_mode] && 0755
-    raise Gem::FilePermissionError, gem_home unless File.writable?(gem_home)
+    FileUtils.mkdir_p gem_home, mode: options[:dir_mode] && 0o755
   end
 
   def verify_spec
-    unless spec.name =~ Gem::Specification::VALID_NAME_PATTERN
+    unless Gem::Specification::VALID_NAME_PATTERN.match?(spec.name)
       raise Gem::InstallError, "#{spec} has an invalid name"
     end
 
@@ -728,11 +734,11 @@ class Gem::Installer
       raise Gem::InstallError, "#{spec} has an invalid extensions"
     end
 
-    if spec.platform.to_s =~ /\R/
+    if /\R/.match?(spec.platform.to_s)
       raise Gem::InstallError, "#{spec.platform} is an invalid platform"
     end
 
-    unless spec.specification_version.to_s =~ /\A\d+\z/
+    unless /\A\d+\z/.match?(spec.specification_version.to_s)
       raise Gem::InstallError, "#{spec} has an invalid specification_version"
     end
 
@@ -749,9 +755,9 @@ class Gem::Installer
   # Return the text for an application file.
 
   def app_script_text(bin_file_name)
-    # note that the `load` lines cannot be indented, as old RG versions match
+    # NOTE: that the `load` lines cannot be indented, as old RG versions match
     # against the beginning of the line
-    return <<-TEXT
+    <<-TEXT
 #{shebang bin_file_name}
 #
 # This file was generated by RubyGems.
@@ -808,10 +814,10 @@ TEXT
     rb_topdir = RbConfig::TOPDIR || File.dirname(rb_config["bindir"])
 
     # get ruby executable file name from RbConfig
-    ruby_exe = "#{rb_config['RUBY_INSTALL_NAME']}#{rb_config['EXEEXT']}"
+    ruby_exe = "#{rb_config["RUBY_INSTALL_NAME"]}#{rb_config["EXEEXT"]}"
     ruby_exe = "ruby.exe" if ruby_exe.empty?
 
-    if File.exist?(File.join bindir, ruby_exe)
+    if File.exist?(File.join(bindir, ruby_exe))
       # stub & ruby.exe within same folder.  Portable
       <<-TEXT
 @ECHO OFF
@@ -933,7 +939,7 @@ TEXT
     build_info_dir = File.join gem_home, "build_info"
 
     dir_mode = options[:dir_mode]
-    FileUtils.mkdir_p build_info_dir, :mode => dir_mode && 0755
+    FileUtils.mkdir_p build_info_dir, mode: dir_mode && 0o755
 
     build_info_file = File.join build_info_dir, "#{spec.full_name}.info"
 
@@ -956,7 +962,7 @@ TEXT
 
   def ensure_writable_dir(dir) # :nodoc:
     begin
-      Dir.mkdir dir, *[options[:dir_mode] && 0755].compact
+      Dir.mkdir dir, *[options[:dir_mode] && 0o755].compact
     rescue SystemCallError
       raise unless File.directory? dir
     end
@@ -966,6 +972,19 @@ TEXT
 
   private
 
+  def user_install_dir
+    # never install to user home in --build-root mode
+    return unless @build_root.nil?
+
+    # Please note that @user_install might have three states:
+    # * `true`: `--user-install`
+    # * `false`: `--no-user-install` and
+    # * `nil`: option was not specified
+    if @user_install || (@user_install.nil? && Gem.default_user_install)
+      Gem.user_dir
+    end
+  end
+
   def build_args
     @build_args ||= begin
                       require_relative "command"
@@ -991,7 +1010,7 @@ TEXT
         bindir="${0%/*}"
       EOS
 
-      script << %Q(exec "$bindir/#{ruby_install_name}" "-x" "$0" "$@"\n)
+      script << %(exec "$bindir/#{ruby_install_name}" "-x" "$0" "$@"\n)
 
       <<~EOS
         #!/bin/sh
